@@ -301,13 +301,13 @@ bamf_view_set_sticky (BamfView *view, gboolean value)
   
   if (value == view->priv->sticky)
     return;
+
+  view->priv->sticky = value;
   
   if (value)
     g_object_ref_sink (view);
   else
     g_object_unref (view);
-  
-  view->priv->sticky = value;
 }
 
 gchar *
@@ -356,7 +356,7 @@ bamf_view_get_name (BamfView *self)
     return BAMF_VIEW_GET_CLASS (self)->get_name (self);
 
   if (!bamf_view_remote_ready (self))
-    return priv->local_name;
+    return g_strdup(priv->local_name);
     
   if (!dbus_g_proxy_call (priv->proxy,
                           "Name",
@@ -556,10 +556,6 @@ bamf_view_on_closed (DBusGProxy *proxy, BamfView *self)
 static void
 bamf_view_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
-  BamfView *self;
-
-  self = BAMF_VIEW (object);
-
   switch (property_id)
     {
       default:
