@@ -17,6 +17,8 @@
  *
  */
 
+#include "config.h"
+
 #include "bamf-legacy-window.h"
 #include "bamf-legacy-screen.h"
 #include <libgtop-2.0/glibtop.h>
@@ -101,6 +103,23 @@ bamf_legacy_window_is_skip_tasklist (BamfLegacyWindow *self)
   return wnck_window_is_skip_tasklist (self->priv->legacy_window);
 }
 
+#ifdef USE_GTK3
+const char *
+bamf_legacy_window_get_class_instance_name (BamfLegacyWindow *self)
+{
+  WnckWindow *window;
+
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  window = self->priv->legacy_window;
+
+  if (!window)
+    return NULL;
+
+  return wnck_window_get_class_instance_name (window);
+}
+#endif
+
 const char *
 bamf_legacy_window_get_class_name (BamfLegacyWindow *self)
 {
