@@ -34,7 +34,12 @@ int main (int argc, char **argv)
   matcher = bamf_matcher_get_default ();
   control = bamf_control_get_default ();
   source = g_object_new (BAMF_TYPE_TAB_SOURCE, "id", "testingsource", NULL);
-  
+
   gtk_main ();
+
+  g_object_unref (matcher);
+  g_object_unref (control);
+  g_object_unref (source);
+
   return 0;
 }
